######################################################################
# DataManip.pm
######################################################################
#
# Perl module for handling reads & writes to our databases
#
#!/usr/local/bin/perl -w

use strict;
use warnings;


# Set up the interface for this package to callers
# Inherit from the Export class
require Exporter;
our @ISA = ("Exporter");

our @EXPORT = qw(); # Symbols to be exported by default
our @EXPORT_OK = qw(read_database display_login); # Export by request

use Fcntl ':flock';

sub read_database();
sub display_login(\$ \$);
sub id_ok(\$);
sub record_submission(\$ \$ \@ \%);
sub display_error_page(\$);
sub resort_websites(\$);

######################################################################
# read_database
######################################################################
# 
# Read the database of already collected websites
#
######################################################################
sub read_database() {
	open(INFH, "<websites.txt") || die("Unable to open database for reading: $!");
	flock(INFH, Fcntl::LOCK_SH) || die("Unable to lock database for reading: $!");
	my %websites = ();
	while(<INFH>) {
		# remove any windows chars and newlines
		$_ =~ s/\r//g;
		chomp();
		my ($index, $category, $name, $url) = split(/\t/);
		$websites{$category}{$name} = [$url, $index];
	}
	close(INFH); 
	return %websites;
}

######################################################################
# display_login
######################################################################
sub display_login(\$ \$) {
	my ($title, $redirect_location) = @_;
    print CGI::header();
    print CGI::start_html($$title), "\n"; 
    print CGI::h1("Enter Participant ID");
    print CGI::start_form('-action' => "$$redirect_location",
			  '-method' => 'POST'),
    "\n";

    print "ID ", CGI::textfield('-name' => 'id', '-size' => 5,
				'-maxlength' => 5), "\n";
    print CGI::submit('-name' => 'id_submit',
		      '-value' => 'Submit'), "\n";
    print CGI::end_form(), "\n";
    print CGI::end_html(), "\n";
}

######################################################################
# id_ok
######################################################################
#
# Double check that we are using one of the valid ID numbers
#
sub id_ok(\$) {
    my ($id_ref) = @_;

    open(IDFILE, "<participantIds.txt") || die ("Unable to read ID database");
    flock(IDFILE, Fcntl::LOCK_SH) || die("Unable to lock ID DB: $!");

    while(<IDFILE>) {
		# remove newlines
		$_ =~ s/\r//g;
		chomp();
		if ($_ == $$id_ref) {
			return 1;
		}
	}    
    close(IDFILE);
    return 0;
}

######################################################################
# record_submission
######################################################################
#
# Extract the params from the last session
#
sub record_submission(\$ \$ \@ \%) {
    my ($id_ref, $cat_ref, $val_ref, $web_ref) = @_;

    my $filename = "responses/$$id_ref.dat";
    open(OUT, ">>$filename") || die "Unable to open file $filename, $!";
	# BUGBUG Get message: Bad file number
    # flock(OUT, Fcntl::LOCK_SH) || die("Unable to lock file $filename: $!");
	# Printout in (Category, Website Title, URL)" form
    foreach my $val (@$val_ref) {
		my $entry = $web_ref->{$$cat_ref}->{$val}; 
		print OUT "($$cat_ref, $val, $entry->[0], $entry->[1])\n";
    }
    close(OUT);
}

######################################################################
# resort_websites
######################################################################
# 
# User has submitted which websites they use.  Now re-sort the websites
# so the user see's a mixed sequence of websites.  This is to prevent
# the user from seeing all websites in one category before moving on
# to the next category so we get a sampling of all of their websites.
#
sub resort_websites(\$) {
	my ($id_ref) = @_;

	# Read the user's indicated list of websites, stored in the
	# file responses/id_number.dat
	my $filename = "responses/$$id_ref.dat";
	open(IN, "<$filename") || die "Unable to open file $filename, $!";
	flock(IN, Fcntl::LOCK_SH) || die("Unable to lock file $filename: $!");
	my %read_sites = ();
	my @submissions = ();

	while(<IN>) {
	    # File was written in the form (category, title, url, index)
	    if(/^\((.*), (.*), (.*), (.*)\)/) {
		my $quad = [$1, $2, $3, $4];
 		unless(defined($read_sites{$2})) {
		    $read_sites{$2} = 1;
		    push(@submissions, $quad);
		}
	    }
	}
	close(IN);

	# Mix up the collection
	for (my $i = 0; $i <= $#submissions; $i++) {
	    my $switchIndex = int(rand($#submissions + 1));
	    my $tmp = $submissions[$i];

	    $submissions[$i] = $submissions[$switchIndex];
	    $submissions[$switchIndex] = $tmp;
	}

	# Store the results
	$filename = "responses/$$id_ref-mixed.dat";
	# Handle the case that the user backed up to add more websites
	# by re-reading the already mixed file
	open(RW_F, "+<$filename"); # Can't reopen the file later
        ## BUGBUG don't have file permissions to write out
	my %completed = ();
	while(<RW_F>) {
	    # File was written in the form (title, url, index, doneBit)
	    if(/^\((.*), (.*), (.*), (.*)\)/) {
		$completed{$1} = $4;
	    }
	}	
	seek(RW_F, 0, 0); # Reset for writing
	foreach my $entry (@submissions) {
	    my $done = 0;
	    # Figure out which ones we've already seen
	    if(defined($completed{$entry->[1]})) {
		$done = $completed{$entry->[1]};
	    }
	    print RW_F "($entry->[1], $entry->[2], $entry->[3], $done)\n";
	}
	close(RW_F);
}

######################################################################
# display_error_page
######################################################################
#
# Print out a webpage with error messages
#
sub display_error_page(\$) {
    my ($error_msg) = @_;
    print CGI::header();
    print CGI::start_html("Unable to continue");
    print CGI::h1("Sorry, we are unable to continue.");
    print "Please report this error to the administrator\n";
    print CGI::center($$error_msg), "\n";
    print CGI::end_html(), "\n";
}


1;
